@@ -4,6 +4,7 @@
 
 #include "loader.h"
 
+#include "application.h"
 #include "buildcfg.h"
 #include "cache.h"
 #include "config.h"
@@ -175,7 +176,6 @@ enum preloader_op {
 /** Image preloader executed in background thread. */
 static void* preloader_thread(__attribute__((unused)) void* data)
 {
-    // size_t index = ctx.cur_idx;
     size_t index = image_list_next_file(ctx.cur_idx);
     const char* source;
     struct image* img;
@@ -248,38 +248,38 @@ static void preloader_ctl(enum preloader_op op)
 
 /**
  * Load first (initial) image.
- * @param start initial index of image in the image list
+ * @param index initial index of image in the image list
  * @param force mandatory image index flag
  * @return loader status
  */
-static enum loader_status load_first(size_t start, bool force)
+static enum loader_status load_first(size_t index, bool force)
 {
     enum loader_status status = ldr_ioerror;
 
-    if (force && start != IMGLIST_INVALID) {
-        const char* source = image_list_get(start);
+    if (force && index != IMGLIST_INVALID) {
+        const char* source = image_list_get(index);
         if (source) {
             struct image* img = loader_load_image(source, &status);
             if (img) {
                 ctx.cur_img = img;
-                ctx.cur_idx = start;
+                ctx.cur_idx = index;
             }
         }
     } else {
-        if (start == IMGLIST_INVALID) {
-            start = 0;
+        if (index == IMGLIST_INVALID) {
+            index = image_list_first();
         }
-        while (start != IMGLIST_INVALID) {
-            const char* source = image_list_get(start);
+        while (index != IMGLIST_INVALID) {
+            const char* source = image_list_get(index);
             if (source) {
                 struct image* img = loader_load_image(source, &status);
                 if (img) {
                     ctx.cur_img = img;
-                    ctx.cur_idx = start;
+                    ctx.cur_idx = index;
                     break;
                 }
             }
-            start = image_list_skip(start);
+            index = image_list_skip(index);
         }
     }
 
@@ -331,7 +331,7 @@ static void on_notify(void)
             pos += sizeof(struct inotify_event) + event->len;
         }
         if (updated) {
-            viewer_reload();
+            app_reload();
         }
     }
 }
@@ -464,17 +464,21 @@ static enum loader_status image_from_exec(struct image* img, const char* cmd)
         return ldr_ioerror;
     }
 
-    if (pid == 0) { // child
-        dup2(pfd[1], STDOUT_FILENO);
-        close(pfd[1]);
-        close(pfd[0]);
-        execlp("sh", "/bin/sh", "-c", cmd, NULL);
-        exit(1);
-    } else { // parent
+    if (pid) { // parent
         close(pfd[1]);
         status = image_from_stream(img, pfd[0]);
         close(pfd[0]);
         waitpid(pid, NULL, 0);
+    } else { // child
+        const char* shell = getenv("SHELL");
+        if (!shell || !*shell) {
+            shell = "/bin/sh";
+        }
+        dup2(pfd[1], STDOUT_FILENO);
+        close(pfd[1]);
+        close(pfd[0]);
+        execlp(shell, shell, "-c", cmd, NULL);
+        exit(1);
     }
 
     return status;
@@ -597,48 +601,17 @@ void loader_create(void)
     config_add_loader(IMGLIST_CFG_SECTION, load_config);
 }
 
-bool loader_init(size_t start, bool force)
+void loader_init(void)
 {
-    enum loader_status status;
-
     cache_init(&ctx.previous, ctx.previous_num);
     cache_init(&ctx.preload, ctx.preload_num);
 
 #ifdef HAVE_INOTIFY
     ctx.notify = inotify_init1(IN_CLOEXEC | IN_NONBLOCK);
     if (ctx.notify >= 0) {
-        ui_add_event(ctx.notify, on_notify);
+        app_watch(ctx.notify, on_notify);
     }
 #endif
-
-    // load the first image
-    status = load_first(start, force);
-    if (status == ldr_success) {
-        preloader_ctl(preloader_start);
-#ifdef HAVE_INOTIFY
-        watch_current();
-#endif
-    } else if (!force) {
-        fprintf(stderr, "No image files found to view, exit\n");
-    } else {
-        const char* reason = "Unknown error";
-        switch (status) {
-            case ldr_success:
-                break;
-            case ldr_unsupported:
-                reason = "Unsupported format";
-                break;
-            case ldr_fmterror:
-                reason = "Invalid format";
-                break;
-            case ldr_ioerror:
-                reason = "I/O error";
-                break;
-        }
-        fprintf(stderr, "%s: %s\n", image_list_get(start), reason);
-    }
-
-    return (status == ldr_success);
 }
 
 void loader_destroy(void)
@@ -646,25 +619,35 @@ void loader_destroy(void)
     preloader_ctl(preloader_stop);
     cache_free(&ctx.previous);
     cache_free(&ctx.preload);
+    image_free(ctx.cur_img);
 }
 
-bool loader_reset(void)
+enum loader_status loader_reset(size_t start, bool force)
 {
-    struct image* img;
+    enum loader_status status = ldr_ioerror;
 
+    // reset
     preloader_ctl(preloader_stop);
     cache_reset(&ctx.previous);
     cache_reset(&ctx.preload);
+    image_free(ctx.cur_img);
+    ctx.cur_img = NULL;
 
-    img = loader_load_image(ctx.cur_img->source, NULL);
-    if (img) {
-        image_free(ctx.cur_img);
-        ctx.cur_img = img;
-        preloader_ctl(preloader_start);
-        return true;
+    // load image
+    if (ctx.cur_img) {
+        status = load_first(ctx.cur_idx, true);
+    } else {
+        status = load_first(start, force);
     }
 
-    return false;
+    if (status == ldr_success) {
+        preloader_ctl(preloader_start);
+#ifdef HAVE_INOTIFY
+        watch_current();
+#endif
+    }
+
+    return status;
 }
 
 struct image* loader_current_image(void)
