@@ -94,6 +94,41 @@ void pixmap_inverse_fill(struct pixmap* pm, ssize_t x, ssize_t y, size_t width,
     }
 }
 
+void pixmap_hline(struct pixmap* pm, ssize_t x, ssize_t y, size_t width,
+                  argb_t color)
+{
+    if (y >= 0 && y < (ssize_t)pm->height) {
+        const ssize_t begin = max(0, x);
+        const ssize_t end = min((ssize_t)pm->width, x + (ssize_t)width);
+        const size_t offset = y * pm->width;
+        for (ssize_t i = begin; i < end; ++i) {
+            alpha_blend(color, &pm->data[offset + i]);
+        }
+    }
+}
+
+void pixmap_vline(struct pixmap* pm, ssize_t x, ssize_t y, size_t height,
+                  argb_t color)
+{
+    if (x >= 0 && x < (ssize_t)pm->width) {
+        const ssize_t begin = max(0, y);
+        const ssize_t end = min((ssize_t)pm->height, y + (ssize_t)height);
+        for (ssize_t i = begin; i < end; ++i) {
+            alpha_blend(color, &pm->data[i * pm->width + x]);
+        }
+    }
+}
+
+void pixmap_rect(struct pixmap* pm, ssize_t x, ssize_t y, size_t width,
+                 size_t height, argb_t color)
+{
+
+    pixmap_hline(pm, x, y, width, color);
+    pixmap_hline(pm, x, y + height - 1, width, color);
+    pixmap_vline(pm, x, y + 1, height - 1, color);
+    pixmap_vline(pm, x + width - 1, y + 1, height - 1, color);
+}
+
 void pixmap_grid(struct pixmap* pm, ssize_t x, ssize_t y, size_t width,
                  size_t height, size_t tail_sz, argb_t color1, argb_t color2)
 {
@@ -163,19 +198,18 @@ void pixmap_copy(const struct pixmap* src, struct pixmap* dst, ssize_t x,
 {
     const ssize_t left = max(0, x);
     const ssize_t top = max(0, y);
-    const ssize_t right = min((ssize_t)dst->width, (ssize_t)src->width + x);
-    const ssize_t bottom = min((ssize_t)dst->height, (ssize_t)src->height + y);
+    const ssize_t right = min((ssize_t)dst->width, x + (ssize_t)src->width);
+    const ssize_t bottom = min((ssize_t)dst->height, y + (ssize_t)src->height);
     const ssize_t dst_width = right - left;
-    const ssize_t dst_height = bottom - top;
+    const ssize_t delta_x = left - x;
+    const ssize_t delta_y = top - y;
     const size_t line_sz = dst_width * sizeof(argb_t);
 
-    if (right < 0 || bottom < 0 || dst_width <= 0 || dst_height <= 0) {
-        return;
-    }
+    for (ssize_t dst_y = top; dst_y < bottom; ++dst_y) {
+        const size_t src_y = dst_y - top + delta_y;
+        const argb_t* src_line = &src->data[src_y * src->width + delta_x];
+        argb_t* dst_line = &dst->data[dst_y * dst->width + left];
 
-    for (y = 0; y < dst_height; ++y) {
-        const argb_t* src_line = &src->data[y * src->width];
-        argb_t* dst_line = &dst->data[(y + top) * dst->width + left];
         if (alpha) {
             for (x = 0; x < dst_width; ++x) {
                 alpha_blend(src_line[x], &dst_line[x]);
