@@ -12,16 +12,14 @@
 // Space between text layout and window edge
 #define TEXT_PADDING 10
 
-/** Text context. */
+/** Text context description. */
 struct text {
     argb_t color;  ///< Font color
     argb_t shadow; ///< Font shadow color
 };
 
-static struct text ctx = {
-    .color = 0xffcccccc,
-    .shadow = 0x80000000,
-};
+/** Global text context. */
+static struct text ctx;
 
 /**
  * Draw text surface on window.
@@ -29,8 +27,8 @@ static struct text ctx = {
  * @param x,y text position
  * @param text text surface to draw
  */
-static void draw_text(struct pixmap* wnd, size_t x, size_t y,
-                      const struct text_surface* text)
+static void put_text(struct pixmap* wnd, size_t x, size_t y,
+                     const struct text_surface* text)
 {
     if (ARGB_GET_A(ctx.shadow)) {
         size_t shadow_offset = text->height / 16;
@@ -72,81 +70,53 @@ static enum config_status load_config(const char* key, const char* value)
 
 void text_create(void)
 {
+    // default settings
+    ctx.color = 0xffcccccc;
+    ctx.shadow = 0x80000000;
+
     // register configuration loader
     config_add_loader(FONT_CONFIG_SECTION, load_config);
 }
 
-void text_print(struct pixmap* wnd, enum info_position pos,
-                const struct info_line* lines, size_t lines_num)
+void text_print(struct pixmap* wnd, enum text_position pos,
+                const struct text_surface* text)
 {
-    size_t max_key_width = 0;
-    const size_t height = lines[0].value.height;
+    size_t x, y;
 
-    // calc max width of keys, used if block on the left side
-    for (size_t i = 0; i < lines_num; ++i) {
-        if (lines[i].key.width > max_key_width) {
-            max_key_width = lines[i].key.width;
-        }
+    // calculate line position
+    switch (pos) {
+        case text_center:
+            x = wnd->width / 2 - text->width / 2;
+            y = wnd->height / 2 - text->height / 2;
+            break;
+        case text_top_left:
+            x = TEXT_PADDING;
+            y = TEXT_PADDING;
+            break;
+        case text_top_right:
+            x = wnd->width - TEXT_PADDING - text->width;
+            y = TEXT_PADDING;
+            break;
+        case text_bottom_left:
+            x = TEXT_PADDING;
+            y = wnd->height - TEXT_PADDING - text->height;
+            break;
+        case text_bottom_right:
+            x = wnd->width - TEXT_PADDING - text->width;
+            y = wnd->height - TEXT_PADDING - text->height;
+            break;
     }
-    max_key_width += height / 2;
 
-    // draw info block
-    for (size_t i = 0; i < lines_num; ++i) {
-        const struct text_surface* key = &lines[i].key;
-        const struct text_surface* value = &lines[i].value;
-        size_t y = 0;
-        size_t x_key = 0;
-        size_t x_val = 0;
-
-        // calculate line position
-        switch (pos) {
-            case info_top_left:
-                y = TEXT_PADDING + i * height;
-                if (key->data) {
-                    x_key = TEXT_PADDING;
-                    x_val = TEXT_PADDING + max_key_width;
-                } else {
-                    x_val = TEXT_PADDING;
-                }
-                break;
-            case info_top_right:
-                y = TEXT_PADDING + i * height;
-                x_val = wnd->width - TEXT_PADDING - value->width;
-                if (key->data) {
-                    x_key = x_val - key->width - TEXT_PADDING;
-                }
-                break;
-            case info_bottom_left:
-                y = wnd->height - TEXT_PADDING - height * lines_num +
-                    i * height;
-                if (key->data) {
-                    x_key = TEXT_PADDING;
-                    x_val = TEXT_PADDING + max_key_width;
-                } else {
-                    x_val = TEXT_PADDING;
-                }
-                break;
-            case info_bottom_right:
-                y = wnd->height - TEXT_PADDING - height * lines_num +
-                    i * height;
-                x_val = wnd->width - TEXT_PADDING - value->width;
-                if (key->data) {
-                    x_key = x_val - key->width - TEXT_PADDING;
-                }
-                break;
-        }
-
-        if (key->data) {
-            draw_text(wnd, x_key, y, key);
-            x_key += key->width;
-        }
-        draw_text(wnd, x_val, y, value);
-    }
+    put_text(wnd, x, y, text);
 }
 
-void text_print_centered(struct pixmap* wnd, const struct text_surface* lines,
-                         size_t lines_num)
+void text_print_lines(struct pixmap* wnd, enum text_position pos,
+                      const struct text_surface* lines, size_t lines_num)
 {
+    if (pos != text_center) {
+        return; // not supported (not used anywhere)
+    }
+
     const size_t line_height = lines[0].height;
     const size_t row_max = (wnd->height - TEXT_PADDING * 2) / line_height;
     const size_t columns =
@@ -190,7 +160,7 @@ void text_print_centered(struct pixmap* wnd, const struct text_surface* lines,
             if (index >= lines_num) {
                 break;
             }
-            draw_text(wnd, left, y, &lines[index]);
+            put_text(wnd, left, y, &lines[index]);
             if (col_width < lines[index].width) {
                 col_width = lines[index].width;
             }
@@ -199,3 +169,73 @@ void text_print_centered(struct pixmap* wnd, const struct text_surface* lines,
         left += col_width + col_space;
     }
 }
+
+void text_print_keyval(struct pixmap* wnd, enum text_position pos,
+                       const struct text_keyval* lines, size_t lines_num)
+{
+    size_t max_key_width = 0;
+    const size_t height = lines[0].value.height;
+
+    // calc max width of keys, used if block on the left side
+    for (size_t i = 0; i < lines_num; ++i) {
+        if (lines[i].key.width > max_key_width) {
+            max_key_width = lines[i].key.width;
+        }
+    }
+    max_key_width += height / 2;
+
+    // draw info block
+    for (size_t i = 0; i < lines_num; ++i) {
+        const struct text_surface* key = &lines[i].key;
+        const struct text_surface* value = &lines[i].value;
+        size_t y = 0;
+        size_t x_key = 0;
+        size_t x_val = 0;
+
+        // calculate line position
+        switch (pos) {
+            case text_center:
+                return; // not supported (not used anywhere)
+            case text_top_left:
+                y = TEXT_PADDING + i * height;
+                if (key->data) {
+                    x_key = TEXT_PADDING;
+                    x_val = TEXT_PADDING + max_key_width;
+                } else {
+                    x_val = TEXT_PADDING;
+                }
+                break;
+            case text_top_right:
+                y = TEXT_PADDING + i * height;
+                x_val = wnd->width - TEXT_PADDING - value->width;
+                if (key->data) {
+                    x_key = x_val - key->width - TEXT_PADDING;
+                }
+                break;
+            case text_bottom_left:
+                y = wnd->height - TEXT_PADDING - height * lines_num +
+                    i * height;
+                if (key->data) {
+                    x_key = TEXT_PADDING;
+                    x_val = TEXT_PADDING + max_key_width;
+                } else {
+                    x_val = TEXT_PADDING;
+                }
+                break;
+            case text_bottom_right:
+                y = wnd->height - TEXT_PADDING - height * lines_num +
+                    i * height;
+                x_val = wnd->width - TEXT_PADDING - value->width;
+                if (key->data) {
+                    x_key = x_val - key->width - TEXT_PADDING;
+                }
+                break;
+        }
+
+        if (key->data) {
+            put_text(wnd, x_key, y, key);
+            x_key += key->width;
+        }
+        put_text(wnd, x_val, y, value);
+    }
+}
