@@ -6,8 +6,9 @@
 
 #include "pixmap.h"
 
+#include <limits.h>
+
 // Configuration parameters
-#define UI_CFG_APP_ID     "app_id"
 #define UI_CFG_FULLSCREEN "fullscreen"
 #define UI_CFG_SIZE       "size"
 #define UI_CFG_POSITION   "position"
@@ -15,39 +16,45 @@
 // Special ids for windows size and position
 #define SIZE_FROM_IMAGE  0
 #define SIZE_FROM_PARENT 1
-#define POS_FROM_PARENT  0xffffffff
+#define POS_FROM_PARENT  SSIZE_MAX
 
 /**
- * Create User Interface context.
+ * Create global UI context.
  */
 void ui_create(void);
 
+/**
+ * Initialize global UI context: create window, register handlers etc.
+ * @param app_id application id, used as window class
+ * @return true if window created
+ */
+bool ui_init(const char* app_id);
+
 /**
  * Destroy global UI context.
  */
 void ui_destroy(void);
 
 /**
- * Run event handler loop.
- * @return true if window was closed by user (not an error)
+ * Prepare the window system to read events.
  */
-bool ui_run(void);
+void ui_event_prepare(void);
 
 /**
- * Stop event handler loop.
+ * Event handler complete notification.
  */
-void ui_stop(void);
+void ui_event_done(void);
 
 /**
- * Redraw window.
+ * Begin window redraw procedure.
+ * @return window pixmap
  */
-void ui_redraw(void);
+struct pixmap* ui_draw_begin(void);
 
 /**
- * Get app id (window class name).
- * @return app id
+ * Finish window redraw procedure.
  */
-const char* ui_get_appid(void);
+void ui_draw_commit(void);
 
 /**
  * Set window title.
@@ -107,13 +114,3 @@ void ui_toggle_fullscreen(void);
  * @return current mode
  */
 bool ui_get_fullscreen(void);
-
-/** Custom event handler. */
-typedef void (*fd_event)(void);
-
-/**
- * Add custom event handler.
- * @param fd file descriptor for polling
- * @param handler callback
- */
-void ui_add_event(int fd, fd_event handler);
