@@ -4,6 +4,7 @@
 
 #include "viewer.h"
 
+#include "application.h"
 #include "buildcfg.h"
 #include "config.h"
 #include "imagelist.h"
@@ -14,12 +15,10 @@
 #include "text.h"
 #include "ui.h"
 
-#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/timerfd.h>
-#include <sys/wait.h>
 #include <unistd.h>
 
 // Background grid parameters
@@ -79,17 +78,8 @@ struct viewer {
     int info_timeout_fd; ///< Info timer
 };
 
-static struct viewer ctx = {
-    .image_bkg = GRID_BKGID,
-    .fixed = true,
-    .animation_enable = true,
-    .animation_fd = -1,
-    .slideshow_enable = false,
-    .slideshow_fd = -1,
-    .slideshow_time = 3,
-    .info_timedout = false,
-    .info_timeout_fd = -1,
-};
+/** Global viewer context. */
+static struct viewer ctx;
 
 /**
  * Fix up image position.
@@ -149,7 +139,7 @@ static void fixup_position(bool force)
  * @param positive direction (increase/decrease)
  * @param params optional move step in percents
  */
-static bool move_image(bool horizontal, bool positive, const char* params)
+static void move_image(bool horizontal, bool positive, const char* params)
 {
     const ssize_t old_x = ctx.img_x;
     const ssize_t old_y = ctx.img_y;
@@ -176,7 +166,9 @@ static bool move_image(bool horizontal, bool positive, const char* params)
 
     fixup_position(false);
 
-    return (ctx.img_x != old_x || ctx.img_y != old_y);
+    if (ctx.img_x != old_x || ctx.img_y != old_y) {
+        app_redraw();
+    }
 }
 
 /**
@@ -194,6 +186,8 @@ static void rotate_image(bool clockwise)
     ctx.img_x += shift;
     ctx.img_y -= shift;
     fixup_position(false);
+
+    app_redraw();
 }
 
 /**
@@ -238,13 +232,14 @@ static void scale_image(enum fixed_scale sc)
     ctx.img_y = wnd_height / 2 - (ctx.scale * pm->height) / 2;
 
     fixup_position(true);
+    app_redraw();
 }
 
 /**
  * Zoom in/out.
  * @param params zoom operation
  */
-void zoom_image(const char* params)
+static void zoom_image(const char* params)
 {
     ssize_t percent = 0;
     ssize_t fixed_scale;
@@ -290,6 +285,8 @@ void zoom_image(const char* params)
     } else {
         fprintf(stderr, "Invalid zoom operation: \"%s\"\n", params);
     }
+
+    app_redraw();
 }
 
 /**
@@ -362,6 +359,8 @@ static void reset_state(void)
         }
         timerfd_settime(ctx.info_timeout_fd, 0, &info_ts, NULL);
     }
+
+    app_redraw();
 }
 
 /**
@@ -425,9 +424,8 @@ static bool next_image(enum action_type direction)
 /**
  * Switch to the next or previous frame.
  * @param forward switch direction
- * @return false if there is only one frame in the image
  */
-static bool next_frame(bool forward)
+static void next_frame(bool forward)
 {
     size_t index = ctx.frame;
     const struct image* img = loader_current_image();
@@ -441,12 +439,10 @@ static bool next_frame(bool forward)
             index = img->num_frames - 1;
         }
     }
-    if (index == ctx.frame) {
-        return false;
+    if (index != ctx.frame) {
+        ctx.frame = index;
+        app_redraw();
     }
-
-    ctx.frame = index;
-    return true;
 }
 
 /**
@@ -456,7 +452,6 @@ static void on_animation_timer(void)
 {
     next_frame(true);
     animation_ctl(true);
-    ui_redraw();
 }
 
 /**
@@ -465,7 +460,6 @@ static void on_animation_timer(void)
 static void on_slideshow_timer(void)
 {
     slideshow_ctl(next_image(action_next_file));
-    ui_redraw();
 }
 
 /**
@@ -477,56 +471,7 @@ static void on_info_block_timeout(void)
     struct itimerspec info_ts = { 0 };
     timerfd_settime(ctx.info_timeout_fd, 0, &info_ts, NULL);
     ctx.info_timedout = true;
-    ui_redraw();
-}
-
-/**
- * Execute system command for the current image.
- * @param expr command expression
- */
-static void execute_command(const char* expr)
-{
-    const char* path = loader_current_image()->source;
-    char* cmd = NULL;
-    int rc = EINVAL;
-
-    // construct command from template
-    while (expr && *expr) {
-        if (*expr == '%') {
-            ++expr;
-            if (*expr != '%') {
-                str_append(path, 0, &cmd); // replace % with path
-                continue;
-            }
-        }
-        str_append(expr, 1, &cmd);
-        ++expr;
-    }
-
-    if (cmd) {
-        rc = system(cmd); // execute
-        if (rc != -1) {
-            rc = WEXITSTATUS(rc);
-        } else {
-            rc = errno ? errno : EINVAL;
-        }
-    }
-
-    // show execution status
-    if (!cmd) {
-        info_set_status("Error: no command to execute");
-    } else {
-        if (strlen(cmd) > 30) { // trim long command
-            strcpy(&cmd[27], "...");
-        }
-        if (rc) {
-            info_set_status("Error %d: %s", rc, cmd);
-        } else {
-            info_set_status("OK: %s", cmd);
-        }
-    }
-
-    free(cmd);
+    app_redraw();
 }
 
 /**
@@ -536,6 +481,8 @@ static void switch_help(void)
 {
     const struct keybind* kb;
 
+    app_redraw();
+
     if (ctx.help) {
         for (size_t i = 0; i < ctx.help_sz; i++) {
             free(ctx.help[i].data);
@@ -606,6 +553,209 @@ static void draw_image(struct pixmap* wnd)
     }
 }
 
+/**
+ * Reload image file and reset state (position, scale, etc).
+ */
+static void reload(void)
+{
+    if (loader_reset(loader_current_index(), false) == ldr_success) {
+        app_status("Image reloaded");
+        reset_state();
+    } else {
+        app_exit(0);
+    }
+}
+
+/**
+ * Redraw handler.
+ */
+static void redraw(void)
+{
+    struct pixmap* window = ui_draw_begin();
+    if (!window) {
+        return;
+    }
+
+    info_update(ctx.frame, ctx.scale);
+    draw_image(window);
+
+    // put text info blocks on window surface
+    if (!ctx.info_timedout) {
+        const enum text_position pos[] = {
+            text_top_left,
+            text_top_right,
+            text_bottom_left,
+            text_bottom_right,
+        };
+        for (size_t i = 0; i < ARRAY_SIZE(pos); ++i) {
+            const size_t lines_num = info_height(pos[i]);
+            if (lines_num) {
+                const struct text_keyval* lines = info_lines(pos[i]);
+                text_print_keyval(window, pos[i], lines, lines_num);
+            }
+        }
+    }
+
+    if (ctx.help) {
+        text_print_lines(window, text_center, ctx.help, ctx.help_sz);
+    }
+
+    // reset one-time rendered notification message
+    app_status(NULL);
+
+    ui_draw_commit();
+}
+
+/**
+ * Window resize handler.
+ */
+static void on_resize(void)
+{
+    fixup_position(false);
+    reset_state();
+}
+
+/**
+ * Key press handler.
+ * @param key code of key pressed
+ * @param mods key modifiers (ctrl/alt/shift)
+ */
+static void on_keyboard(xkb_keysym_t key, uint8_t mods)
+{
+    const struct keybind* kb = keybind_get(key, mods);
+
+    if (!kb) {
+        char* name = keybind_name(key, mods);
+        if (name) {
+            app_status("Key %s is not bound", name);
+            free(name);
+            app_redraw();
+        }
+        return;
+    }
+
+    // handle actions
+    for (size_t i = 0; i < kb->num_actions; ++i) {
+        const struct action* action = &kb->actions[i];
+        switch (action->type) {
+            case action_none:
+                break;
+            case action_help:
+                switch_help();
+                break;
+            case action_first_file:
+            case action_last_file:
+            case action_prev_dir:
+            case action_next_dir:
+            case action_prev_file:
+            case action_next_file:
+                next_image(action->type);
+                break;
+            case action_skip_file:
+                if (!skip_image()) {
+                    printf("No more images, exit\n");
+                    app_exit(0);
+                    return;
+                }
+                reset_state();
+                break;
+            case action_prev_frame:
+            case action_next_frame:
+                animation_ctl(false);
+                next_frame(action->type == action_next_frame);
+                break;
+            case action_animation:
+                animation_ctl(!ctx.animation_enable);
+                break;
+            case action_slideshow:
+                slideshow_ctl(!ctx.slideshow_enable &&
+                              next_image(action_next_file));
+                break;
+            case action_fullscreen:
+                ui_toggle_fullscreen();
+                break;
+            case action_mode:
+                app_switch_mode();
+                break;
+            case action_step_left:
+                move_image(true, true, action->params);
+                break;
+            case action_step_right:
+                move_image(true, false, action->params);
+                break;
+            case action_step_up:
+                move_image(false, true, action->params);
+                break;
+            case action_step_down:
+                move_image(false, false, action->params);
+                break;
+            case action_zoom:
+                zoom_image(action->params);
+                break;
+            case action_rotate_left:
+                rotate_image(false);
+                break;
+            case action_rotate_right:
+                rotate_image(true);
+                break;
+            case action_flip_vertical:
+                image_flip_vertical(loader_current_image());
+                app_redraw();
+                break;
+            case action_flip_horizontal:
+                image_flip_horizontal(loader_current_image());
+                app_redraw();
+                break;
+            case action_antialiasing:
+                ctx.antialiasing = !ctx.antialiasing;
+                app_status("Anti-aliasing %s", ctx.antialiasing ? "on" : "off");
+                app_redraw();
+                break;
+            case action_reload:
+                reload();
+                break;
+            case action_info:
+                info_set_mode(action->params);
+                app_redraw();
+                break;
+            case action_exec:
+                app_execute(action->params, loader_current_image()->source);
+                break;
+            case action_status:
+                app_status("%s", action->params);
+                app_redraw();
+                break;
+            case action_exit:
+                if (ctx.help) {
+                    switch_help(); // remove help overlay
+                } else {
+                    app_exit(0);
+                    return;
+                }
+                break;
+        }
+        ++action;
+    }
+}
+
+/**
+ * Image drag handler.
+ * @param dx,dy delta to move viewpoint
+ */
+static void on_drag(int dx, int dy)
+{
+    const ssize_t old_x = ctx.img_x;
+    const ssize_t old_y = ctx.img_y;
+
+    ctx.img_x += dx;
+    ctx.img_y += dy;
+
+    if (ctx.img_x != old_x || ctx.img_y != old_y) {
+        fixup_position(false);
+        app_redraw();
+    }
+}
+
 /**
  * Custom section loader, see `config_loader` for details.
  */
@@ -657,6 +807,17 @@ static enum config_status load_config(const char* key, const char* value)
 
 void viewer_create(void)
 {
+    // set default configuration
+    ctx.image_bkg = GRID_BKGID;
+    ctx.fixed = true;
+    ctx.animation_enable = true;
+    ctx.animation_fd = -1;
+    ctx.slideshow_enable = false;
+    ctx.slideshow_fd = -1;
+    ctx.slideshow_time = 3;
+    ctx.info_timedout = false;
+    ctx.info_timeout_fd = -1;
+
     // register configuration loader
     config_add_loader(GENERAL_CONFIG_SECTION, load_config);
 }
@@ -667,14 +828,14 @@ void viewer_init(void)
     ctx.animation_fd =
         timerfd_create(CLOCK_MONOTONIC, TFD_CLOEXEC | TFD_NONBLOCK);
     if (ctx.animation_fd != -1) {
-        ui_add_event(ctx.animation_fd, on_animation_timer);
+        app_watch(ctx.animation_fd, on_animation_timer);
     }
 
     // setup slideshow timer
     ctx.slideshow_fd =
         timerfd_create(CLOCK_MONOTONIC, TFD_CLOEXEC | TFD_NONBLOCK);
     if (ctx.slideshow_fd != -1) {
-        ui_add_event(ctx.slideshow_fd, on_slideshow_timer);
+        app_watch(ctx.slideshow_fd, on_slideshow_timer);
     }
 
     // setup info block timer
@@ -682,7 +843,7 @@ void viewer_init(void)
         ctx.info_timeout_fd =
             timerfd_create(CLOCK_MONOTONIC, TFD_CLOEXEC | TFD_NONBLOCK);
         if (ctx.info_timeout_fd != -1) {
-            ui_add_event(ctx.info_timeout_fd, on_info_block_timeout);
+            app_watch(ctx.info_timeout_fd, on_info_block_timeout);
         }
     }
 }
@@ -703,189 +864,27 @@ void viewer_destroy(void)
     }
 }
 
-void viewer_reload(void)
+void viewer_handle(const struct event* event)
 {
-    if (loader_reset()) {
-        info_set_status("Image reloaded");
-    } else if (skip_image()) {
-        info_set_status("Image skipped due to reload error");
-    } else {
-        info_set_status("Unable to reload image");
-    }
-    reset_state();
-    ui_redraw();
-}
-
-void viewer_on_redraw(struct pixmap* window)
-{
-    info_update(ctx.frame, ctx.scale);
-    draw_image(window);
-
-    // put text info blocks on window surface
-    if (!ctx.info_timedout) {
-        for (size_t i = 0; i < INFO_POSITION_NUM; ++i) {
-            const size_t lines_num = info_height(i);
-            if (lines_num) {
-                const enum info_position pos = (enum info_position)i;
-                const struct info_line* lines = info_lines(pos);
-                text_print(window, pos, lines, lines_num);
-            }
-        }
-    }
-
-    if (ctx.help) {
-        text_print_centered(window, ctx.help, ctx.help_sz);
-    }
-
-    // reset one-time rendered notification message
-    info_set_status(NULL);
-}
-
-void viewer_on_resize(void)
-{
-    fixup_position(false);
-    reset_state();
-}
-
-void viewer_on_keyboard(xkb_keysym_t key, uint8_t mods)
-{
-    bool redraw = false;
-    const struct keybind* kb = keybind_get(key, mods);
-
-    if (!kb) {
-        char* name = keybind_name(key, mods);
-        if (name) {
-            info_set_status("Key %s is not bound", name);
-            free(name);
-            ui_redraw();
-        }
-        return;
-    }
-
-    // handle actions
-    for (size_t i = 0; i < kb->num_actions; ++i) {
-        const struct action* action = &kb->actions[i];
-        switch (action->type) {
-            case action_none:
-                break;
-            case action_help:
-                switch_help();
-                redraw = true;
-                break;
-            case action_first_file:
-            case action_last_file:
-            case action_prev_dir:
-            case action_next_dir:
-            case action_prev_file:
-            case action_next_file:
-                redraw |= next_image(action->type);
-                break;
-            case action_skip_file:
-                if (!skip_image()) {
-                    printf("No more images, exit\n");
-                    ui_stop();
-                    return;
-                }
-                reset_state();
-                redraw = true;
-                break;
-            case action_prev_frame:
-            case action_next_frame:
-                animation_ctl(false);
-                redraw |= next_frame(action->type == action_next_frame);
-                break;
-            case action_animation:
-                animation_ctl(!ctx.animation_enable);
-                break;
-            case action_slideshow:
-                slideshow_ctl(!ctx.slideshow_enable &&
-                              next_image(action_next_file));
-                redraw = true;
-                break;
-            case action_fullscreen:
-                ui_toggle_fullscreen();
-                break;
-            case action_step_left:
-                redraw |= move_image(true, true, action->params);
-                break;
-            case action_step_right:
-                redraw |= move_image(true, false, action->params);
-                break;
-            case action_step_up:
-                redraw |= move_image(false, true, action->params);
-                break;
-            case action_step_down:
-                redraw |= move_image(false, false, action->params);
-                break;
-            case action_zoom:
-                zoom_image(action->params);
-                redraw = true;
-                break;
-            case action_rotate_left:
-                rotate_image(false);
-                redraw = true;
-                break;
-            case action_rotate_right:
-                rotate_image(true);
-                redraw = true;
-                break;
-            case action_flip_vertical:
-                image_flip_vertical(loader_current_image());
-                redraw = true;
-                break;
-            case action_flip_horizontal:
-                image_flip_horizontal(loader_current_image());
-                redraw = true;
-                break;
-            case action_antialiasing:
-                ctx.antialiasing = !ctx.antialiasing;
-                info_set_status("Anti-aliasing %s",
-                                ctx.antialiasing ? "on" : "off");
-                redraw = true;
-                break;
-            case action_reload:
-                viewer_reload();
-                break;
-            case action_info:
-                info_set_mode(action->params);
-                redraw = true;
-                break;
-            case action_exec:
-                execute_command(action->params);
-                redraw = true;
-                break;
-            case action_status:
-                info_set_status("%s", action->params);
-                redraw = true;
-                break;
-            case action_exit:
-                if (ctx.help) {
-                    switch_help(); // remove help overlay
-                    redraw = true;
-                } else {
-                    ui_stop();
-                    return;
-                }
-                break;
-        }
-        ++action;
-    }
-
-    if (redraw) {
-        ui_redraw();
-    }
-}
-
-void viewer_on_drag(int dx, int dy)
-{
-    const ssize_t old_x = ctx.img_x;
-    const ssize_t old_y = ctx.img_y;
-
-    ctx.img_x += dx;
-    ctx.img_y += dy;
-
-    if (ctx.img_x != old_x || ctx.img_y != old_y) {
-        fixup_position(false);
-        ui_redraw();
+    switch (event->type) {
+        case event_reload:
+            reload();
+            break;
+        case event_redraw:
+            redraw();
+            break;
+        case event_resize:
+            on_resize();
+            break;
+        case event_keypress:
+            on_keyboard(event->param.keypress.key, event->param.keypress.mods);
+            break;
+        case event_drag:
+            on_drag(event->param.drag.dx, event->param.drag.dy);
+            break;
+        case event_activate:
+            reset_state();
+            app_redraw();
+            break;
     }
 }
