@@ -4,8 +4,7 @@
 
 #pragma once
 
-#include "keybind.h"
-#include "pixmap.h"
+#include "event.h"
 
 // Configuration parameters
 #define VIEWER_CFG_SCALE          "scale"
@@ -32,30 +31,6 @@ void viewer_init(void);
 void viewer_destroy(void);
 
 /**
- * Reset state: reload image file, set initial scale etc.
+ * Event handler, see `event_handler` for details.
  */
-void viewer_reload(void);
-
-/**
- * Redraw handler.
- * @param window pixel map of window
- */
-void viewer_on_redraw(struct pixmap* window);
-
-/**
- * Window resize handler.
- */
-void viewer_on_resize(void);
-
-/**
- * Key press handler.
- * @param key code of key pressed
- * @param mods key modifiers (ctrl/alt/shift)
- */
-void viewer_on_keyboard(xkb_keysym_t key, uint8_t mods);
-
-/**
- * Image drag handler.
- * @param dx,dy delta to move viewpoint
- */
-void viewer_on_drag(int dx, int dy);
+void viewer_handle(const struct event* event);
